local addon = PvPTimer
local L = addon.Locale
local LSM = addon.Lib.Media
local cdf = addon.Lib.CDFrames

local CopyTable = addon.CopyTable

-- generic getter/setter functions, use AceGUI's IDs to get/set values
local function getConfigValue(i)
	local db = addon.DB.profile
	
	for k = 1, #i do
		if not db then db = {} end
		db = db[i[k]]
	end
	
	return db
end

local function setConfigValue(i, v)
	local db = addon.DB.profile
	
	for k = 1, #i-1 do
		if not db[i[k]] then db[i[k]] = {} end
		db = db[i[k]]
	end
	
	db[i[#i]] = v
end

-- for colors
local function getColorValue(i)
	local t = getConfigValue(i)
	
	if t then
		return t.r, t.g, t.b, t.a
	else
		return 0, 0, 0, 1
	end
end

local function setColorValue(i, r, g, b, a)
	local t = {
		["r"] = r,
		["g"] = g,
		["b"] = b,
		["a"] = a,
	}
	setConfigValue(i, t)
end

-- for multiselect controls
local function getMultiValue(i, k)
	i[#i+1] = k
	return getConfigValue(i)
end

local function setMultiValue(i, k, v)
	i[#i+1] = k
	setConfigValue(i, v)
end

local const_bg = addon.Const.Battlegrounds
local const_arena = addon.Const.Arenas
local const_autohide = addon.Const.Autohide
local const_alerttarget = addon.Const.AlertTarget
local const_attach = addon.Const.Attach
local const_justify = addon.Const.Justify

-- option subtables
local alert = {
	CautionHeader = {
		order = 1,
		name = L["CAUTION"],
		type = "header",
	},
	CautionText = {
		order = 2,
		name = L["Be careful when using these alerts, they can send out a lot of messages. People will likely yell at you or /ignore you if you spam alerts in a public channel."],
		type = "description",
		width = "full",
		fontSize = "medium",
	},
	EnabledHeader = {
		order = 10,
		name = L["Enable Alert"],
		type = "header",
	},
	Enabled = {
		order = 11,
		name = L["Enable alerts"],
		desc = L["Enable or disable alerts regardless of other settings."],
		type = "toggle",
	},
	ZoneText = {
		order = 20,
		name = L["Alert Target"],
		type = "header",
	},
	BG = {
		order = 21,
		name = L["Battlegrounds"],
		desc = L["Enable or disable in normal, non-rated Battlegrounds."],
		type = "select",
		values = const_alerttarget,
	},
	BG_OnlyTarget = {
		order = 22,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	RatedBG = {
		order = 31,
		name = L["Rated Battlegrounds"],
		desc = L["Enable or disable in rated Battlegrounds."],
		type = "select",
		values = const_alerttarget,
	},
	RatedBG_OnlyTarget = {
		order = 32,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	Arena = {
		order = 41,
		name = "Arenas",
		desc = L["Enable or disable in arenas."],
		type = "select",
		values = const_alerttarget,
	},
	Arena_OnlyTarget = {
		order = 42,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
	Else = {
		order = 98,
		name = L["Everywhere else"],
		desc = L["Enable or disable anywhere else not specified above."],
		type = "select",
		values = const_alerttarget,
	},
	Else_OnlyTarget = {
		order = 99,
		name = L["Only target/focus"],
		desc = L["Only trigger alerts if source is the current target or focus."],
		type = "toggle",
	},
}

local anchor = {
	Enabled = {
		order = 11,
		name = L["Enable anchor"],
		desc = L["Enables or disables this anchor."],
		type = "toggle",
	},
	-- Locked = {
		-- order = 12,
		-- name = L["Lock anchor"],
		-- desc = L["Locks or unlocks this anchor so it can be dragged around and resized using the mouse."],
		-- type = "toggle",
		-- set = function(i, v)
			-- cdf:UnlockAnchor(addon.Anchors[i[#i-1]], not v)
			-- setConfigValue(i, v)
		-- end,
	-- },
	Autohide = {
		order = 13,
		name = L["If target is not an enemy player"],
		desc = L["Behavior when target is not a valid enemy player."],
		type = "select",
		-- style = "radio",
		values = const_autohide,
	},
	Width = {
		order = 21,
		name = L["Width"],
		desc = L["Width of the anchor."],
		type = "range",
		min = 100,
		max = 400,
		step = 5,
		width = "full",
	},
	Scale = {
		order = 22,
		name = L["Scale"],
		desc = L["Scale of the anchor."],
		type = "range",
		min = 0.1,
		max = 5,
		step = 0.05,
		isPercent = true,
		width = "full",
	},
	Text_Enable = {
		order = 50,
		name = L["Show anchor in"],
		type = "header",
		width = "full",
	},
	BG = {
		order = 51,
		name = L["Battlegrounds"],
		desc = L["Enable or disable in normal, non-rated Battlegrounds."],
		type = "toggle",
	},
	RatedBG = {
		order = 52,
		name = L["Rated Battlegrounds"],
		desc = L["Enable or disable in rated Battlegrounds."],
		type = "toggle",
	},
	Arena = {
		order = 53,
		name = L["Arenas"],
		desc = L["Enable or disable in arenas."],
		type = "toggle",
	},
	-- Duel = {
		-- order = 54,
		-- name = "Duels",
		-- type = "toggle",
	-- },
	-- PvP = {
		-- order = 55,
		-- name = "PvP flagged",
		-- type = "toggle",
	-- },
	Else = {
		order = 59,
		name = L["Everywhere else"],
		desc = L["Enable or disable anywhere else not specified above."],
		type = "toggle",
	},
	Position = {
		order = 10,
		name = L["Position"],
		desc = L["Position of the anchor."],
		type = "group",
		args = {
			AttachPoint = {
				order = 11,
				name = L["Anchor point"],
				desc = L["Sets the point that will be anchored."],
				type = "select",
				values = const_attach,
			},
			AttachFrame = {
				order = 12,
				name = L["Relative to"],
				desc = L["Sets the frame to be anchored to.\nUse 'UIParent' for the whole screen."],
				type = "input",
				set = function(i, v)
					if not _G[v] or not _G[v].SetPoint then
						v = "UIParent"
						addon:Print(L["Invalid frame. Resetting to default."])
					end
					setConfigValue(i, v)
					addon:RefreshAnchors()
				end,
			},
			AttachRelative = {
				order = 13,
				name = L["Relative point"],
				desc = L["Sets the point to be anchored to."],
				type = "select",
				values = const_attach,
			},
			AttachX = {
				order = 14,
				name = L["Offset X"],
				desc = L["Horizontal offset."],
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
			AttachY = {
				order = 15,
				name = L["Offset Y"],
				desc = L["Vertical offset."],
				type = "range",
				softMin = -1000,
				softMax = 1000,
				min = -2000,
				max = 2000,
				step = 1,
				bigStep = 5,
				width = "full",
			},
		},
	},
	Header = {
		order = 20,
		name = L["Header"],
		desc = L["Header settings."],
		type = "group",
		args = {
			-- CopySettings = {
				-- order = 1,
				-- name = "Copy settings",
				-- type = "execute",
			-- },
			-- PasteSettings = {
				-- order = 2,
				-- name = "Paste settings",
				-- type = "execute",
				-- disabled = function()
					-- if addon.CopyHeaderSettings == nil then
						-- return true
					-- else
						-- return false
					-- end
				-- end,
			-- },
			Show = {
				order = 1,
				name = L["Show header"],
				desc = L["Shows or hides the header part."],
				type = "toggle",
			},
			ShowClass = {
				order = 11,
				name = L["Show class icon"],
				desc = L["Shows the class icon on the header's left side."],
				type = "toggle",
			},
			ShowSpec = {
				order = 12,
				name = L["Show spec icon"],
				desc = L["Shows the talent specialization icon (or a question mark if unknown) on the header's right side."],
				type = "toggle",
			},
			Color = {
				order = 13,
				name = L["Header color"],
				desc = L["The header's background color."],
				type = "color",
				hasAlpha = true,
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			ClassColor = {
				order = 14,
				name = L["Use class color"],
				desc = L["Uses class color instead of default color."],
				type = "toggle",
			},
			Texture = {
				order = 15,
				name = L["Header texture"],
				desc = L["The header's background texture."],
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = AceGUIWidgetLSMlists.statusbar,
			},
			Font = {
				order = 16,
				name = L["Header font"],
				desc = L["Font used to display target's name."],
				type = "select",
				dialogControl = 'LSM30_Font',
				values = AceGUIWidgetLSMlists.font,
			},
			FontSize = {
				order = 17,
				name = L["Header font size"],
				desc = L["Size of the font."],
				type = "range",
				min = 1,
				max = 20,
				step = 1,
			},
			Justify = {
				order = 18,
				name = L["Justification"],
				desc = L["Justification of the name."],
				type = "select",
				values = const_justify,
			},
		},
	},
	Bars = {
		order = 30,
		name = L["Timer bars"],
		desc = L["Timer bar settings."],
		type = "group",
		args = {
			-- CopySettings = {
				-- order = 1,
				-- name = "Copy settings",
				-- type = "execute",
			-- },
			-- PasteSettings = {
				-- order = 2,
				-- name = "Paste settings",
				-- type = "execute",
				-- disabled = function()
					-- if addon.CopyBarSettings == nil then
						-- return true
					-- else
						-- return false
					-- end
				-- end,
			-- },
			Color = {
				order = 11,
				name = L["Bar color"],
				desc = L["Default timer bar color."],
				type = "color",
				hasAlpha = true,
				get = getColorValue,
				set = function(i, r, g, b, a)
					setColorValue(i, r, g, b, a)
					addon:RefreshAnchors()
				end,
			},
			TypeColor = {
				order = 12,
				name = L["Use spell category color"],
				desc = L["Use spell category color (defined in Spell Categories) instead."],
				type = "toggle",
			},
			GrowUp = {
				order = 13,
				name = L["Bars grow up"],
				desc = L["New bars are added upwards instead of downwards."],
				type = "toggle",
			},
			Columns = {
				order = 14,
				name = L["Columns"],
				desc = L["Arrange bars into columns."],
				type = "range",
				min = 1,
				max = 5,
				step = 1,
			},
			Texture = {
				order = 15,
				name = L["Bar texture"],
				desc = L["Texture of timer bars."],
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = AceGUIWidgetLSMlists.statusbar,
			},
			Text_SpellName = {
				order = 20,
				name = L["Spell Name"],
				type = "header",
			},
			ShowName = {
				order = 21,
				name = L["Show spell name"],
				desc = L["Show the spell's name on the timer bar."],
				type = "toggle",
				width = "full",
			},
			-- NameFont = {
				-- order = 22,
				-- name = L["Font"],
				-- type = "select",
				-- dialogControl = 'LSM30_Font',
				-- values = AceGUIWidgetLSMlists.font,
			-- },
			-- NameFontSize = {
				-- order = 23,
				-- name = L["Font size"],
				-- type = "range",
				-- min = 1,
				-- max = 20,
				-- step = 1,
			-- },
			Text_Time = {
				order = 30,
				name = L["Time Remaining"],
				type = "header",
			},
			ShowTime = {
				order = 31,
				name = L["Show time remaining"],
				desc = L["Show remaining time on the timer bar."],
				type = "toggle",
				width = "full",
			},
			-- TimeFont = {
				-- order = 32,
				-- name = L["Font"],
				-- type = "select",
				-- dialogControl = 'LSM30_Font',
				-- values = AceGUIWidgetLSMlists.font,
			-- },
			-- TimeFontSize = {
				-- order = 33,
				-- name = L["Font size"],
				-- type = "range",
				-- min = 1,
				-- max = 20,
				-- step = 1,
			-- },
		},
	},
}

local anchor_arena = addon:CopyTable(anchor)
anchor_arena.Autohide = nil
anchor_arena.Text_Enable = nil
anchor_arena.BG = nil
anchor_arena.RatedBG = nil
anchor_arena.Arena = nil
anchor_arena.Duel = nil
anchor_arena.PvP = nil
anchor_arena.Else = nil

anchor_arena.Position.args.NextX = {
	order = 18,
	name = L["Distance between frames X"],
	desc = L["Horizontal distance between each arena frame."],
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}
anchor_arena.Position.args.NextY = {
	order = 19,
	name = L["Distance between frames Y"],
	desc = L["Vertical distance between each arena frame."],
	type = "range",
	softMin = -1000,
	softMax = 1000,
	min = -2000,
	max = 2000,
	step = 1,
	bigStep = 5,
	width = "full",
}

local spellcat = {
	offensive = {
		order = 101,
		type = "color",
		name = L["Offensive CDs"],
		desc = L["Cooldowns that increase damage output, e.g. Recklessness."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	defensive = {
		order = 102,
		type = "color",
		name = L["Defensive CDs"],
		desc = L["Defensive and healing cooldowns, e.g. Shield Wall or Swiftmend."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	interrupt = {
		order = 103,
		type = "color",
		name = L["Interrupts and Silences"],
		desc = L["Abilities that interrupt spellcasting or silence, e.g. Kick."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	cc = {
		order = 104,
		type = "color",
		name = L["Crowd Control"],
		desc = L["Abilities that make you lose control of your character, e.g. Fear or Polymorph."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	root = {
		order = 105,
		type = "color",
		name = L["Roots and Snares"],
		desc = L["Spells that restrict or prevent movement, e.g. Frost Nova or Hamstring."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
	misc = {
		order = 106,
		type = "color",
		name = L["Miscellaneous Spells"],
		desc = L["Anything else. Spells that don't really fit into other categories."],
		get = getColorValue,
		set = function(i, r, g, b, a)
			setColorValue(i, r, g, b, a)
		end,
	},
}

addon.EmptyOptions = {
	type = "group",
	name = "PvPTimer",
	args = {
	},
}

--main option table
addon.Options = {
	type = "group",
	name = "PvPTimer",
	get = getConfigValue,
	set = setConfigValue,
	args = {
		Settings = {
			order = 1,
			name = L["Settings"],
			type = "group",
			args = {
				EnableAddon = {
					order = 1,
					name = L["Enable addon"],
					desc = L["Enables or disables the whole addon."],
					type = "toggle",
					set = function(i, v)
						setConfigValue(i, v)
						addon:ZONE_CHANGED_NEW_AREA();
					end,
				},
				OpenSpellConfig = {
					order = 2,
					name = L["Spell Configuration"],
					desc = L["Opens the Spell Configuration window, where you can disable and adjust spells."],
					type = "execute",
					func = function() addon:ToggleSpellConfig() end,
				},
				Generate = {
					cmdHidden = true,
					order = 99,
					name = L["Create options"],
					desc = L["Option tables have not yet been created to reduce memory footprint. \n\nClick here to create the options table."],
					type = "execute",
					func = function() addon:CreateBlizOptions(); GameTooltip:Hide() end,
					hidden = function() return addon.OptionsGenerated end,
					width = "full",
				},
			},
		},
		Anchors = {
			order = 2,
			name = "",
			type = "group",
			set = function(i, v)
				setConfigValue(i, v)
				addon:RefreshAnchors()
			end,
			args = {
				Anchor_Target = {
					order = 2,
					name = L["Target"],
					desc = L["Anchor that shows information about your current target."],
					type = "group",
					args = anchor,
				},
				Anchor_Focus = {
					order = 3,
					name = L["Focus"],
					desc = L["Anchor that shows information about your current focus target."],
					type = "group",
					args = anchor,
				},
				Anchor_Arena = {
					order = 4,
					name = L["Arena Targets"],
					desc = L["Anchor that shows information about arena opponents."],
					type = "group",
					args = anchor_arena,
				},
			},
		},
		Alert = {
			order = 3,
			name = "",
			type = "group",
			args = {
				offensive = {
					order = 101,
					type = "group",
					name = L["Offensive CDs"],
					desc = L["Cooldowns that increase damage output, e.g. Recklessness."],
					args = alert,
				},
				defensive = {
					order = 102,
					type = "group",
					name = L["Defensive CDs"],
					desc = L["Defensive and healing cooldowns, e.g. Shield Wall or Swiftmend."],
					args = alert,
				},
				interrupt = {
					order = 103,
					type = "group",
					name = L["Interrupts and Silences"],
					desc = L["Abilities that interrupt spellcasting or silence, e.g. Kick."],
					args = alert,
				},
				cc = {
					order = 104,
					type = "group",
					name = L["Crowd Control"],
					desc = L["Abilities that make you lose control of your character, e.g. Fear or Polymorph."],
					args = alert,
				},
				root = {
					order = 105,
					type = "group",
					name = L["Roots and Snares"],
					desc = L["Spells that restrict or prevent movement, e.g. Frost Nova or Hamstring."],
					args = alert,
				},
				misc = {
					order = 106,
					type = "group",
					name = L["Miscellaneous Spells"],
					desc = L["Anything else. Spells that don't really fit into other categories."],
					args = alert,
				},
				spec = {
					order = 999,
					type = "group",
					name = L["Spec Detection"],
					desc = L["Talent spec detection alerts."],
					args = alert,
				},
			},
		},
		SpellCategories = {
			order = 4,
			name = "",
			type = "group",
			args = spellcat,
		},
	},
}
