local addon = PvPTimer
local L = addon.Locale

local bband = bit.band
local smatch = string.match

function addon:ConvertTime(number)
	if not number or number == 0 then
		return L["none"]
	elseif math.abs(number) < 60 then
		return number.." "..L["sec"]
	elseif (math.abs(number) % 60)>0 then
		return math.floor(number/60).." "..L["min"].." "..(number%60).." "..L["sec"]
	else
		return math.floor(number/60).." "..L["min"]
	end
end

function addon:CopyTable(t)
	if type(t) ~= "table" then
		return t
	end
	local new = {}
	local i, v = next(t, nil)
	while i do
		if type(v) == "table" then 
			v = CopyTable(v)
		end 
		new[i] = v
		i, v = next(t, i)
	end
	return new
end

-- returns spell data for the specified ID
function addon:GetSpell(spellID)
	local spells = addon.Spells
	
	-- is it in the database?
	if not spells[spellID] then return false end
	-- is it turned off?
	local db = addon.DB.profile.Spells
	if db[spellID] and db[spellID].enabled == false then return false end

	-- get real data if it's a pointer
	if type(spells[spellID]) ~= "table" then
		spellID = spells[spellID]
	end
	
	return spells[spellID]
end

-- calculates spell cooldown with glyphs and talents
function addon:GetSpellCooldown(spellID, glyphed, spec)
	local spell = addon:GetSpell(spellID)
	if not spell then return 0 end
	
	local cd = spell.cooldown
	if not cd or cd == 0 then return 0 end
	
	if spell.cooldown_g and glyphed then
		cd = cd + spell.cooldown_g
	end
	
	if spell.cooldown_s1 and spec == 1 then
		cd = cd + spell.cooldown_s1
	end
	if spell.cooldown_s2 and spec == 2 then
		cd = cd + spell.cooldown_s2
	end
	if spell.cooldown_s3 and spec == 3 then
		cd = cd + spell.cooldown_s3
	end
	
	return cd
end

-- check if glyph is turned on for this spell and spec in spell config
function addon:GetGlyphSettings(spellID, spec)
	local db = addon.DB.profile.Spells
	local result = false
	
	if db[spellID] then
		if db[spellID].glyph0 and spec == 0 then
			result = true
		elseif db[spellID].glyph1 and spec == 1 then
			result = true
		elseif db[spellID].glyph2 and spec == 2 then
			result = true
		elseif db[spellID].glyph3 and spec == 3 then
			result = true
		end
	end
	
	return result
end

-- gets timer bar color based on spell category or default anchor settings
function addon:GetBarColor(anchor, spelltype)
	local db = addon.DB.profile
	local barcolor

	if db.Anchors[anchor].Bars.TypeColor then
		barcolor = db.SpellCategories[spelltype]
	else
		barcolor = db.Anchors[anchor].Bars.Color
	end
	
	return barcolor
end

-- returns true if unit is hostile to player
function addon:IsHostile(flags)
	if not flags then return false end
	return (bband(flags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE)
end

-- strips server from unit name
function addon:RemoveServer(unitname)
	local name, server = smatch(unitname, "(.-)%-(.*)$")
	if not name or not server then
		return unitname
	end
	
	return name
end

function addon:GetZoneSetting(key, subkey)
	local db = addon.DB.profile
	local zonetype = select(2, IsInInstance())
	local rated = IsRatedBattleground()
	
	-- is it enabled?
	if db[key][subkey].Enabled == false then return false end

	if zonetype == "pvp" and not rated then -- we're in a normal bg
		return db[key][subkey].BG
	elseif zonetype == "pvp" and rated then -- we're in a rated bg
		return db[key][subkey].RatedBG
	elseif zonetype == "arena" then -- we're in an arena
		return db[key][subkey].Arena
	else -- we're somewhere else
		return db[key][subkey].Else
	end
end